/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package mathutil;

/**
 * <p>Tda slou k uchovn vektoru (ady) komplexnch sel.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class ComplexVector {
  private Complex[] num;
  private int length;

  /**
   * Inicializuje vektor na danou dlku.
   * @param length int
   */
  public ComplexVector(int length) {
    this.length = length;
    num = new Complex[length];
  }

  public ComplexVector(ComplexVector cv) {
    this.length = cv.length;
    num = new Complex[length];
    for(int i = length-1; i >= 0; i--) {
      this.num[i] = new Complex(cv.num[i]);
    }
  }

  public ComplexVector(Vektor v) {
    this.length = v.getLength();
    num = new Complex[length];
    for(int i = length-1; i >= 0; i--) {
      this.num[i] = new Complex(v.get(i));
    }
  }

  public Complex get(int i) {
    return num[i];
  }

  public void set(int i, Complex c) {
    num[i] = c;
  }

  public int length() {
    return length;
  }

  /**
   * Vrt st vektoru komplexnch sel
   * @param start int poten index (vetn)
   * @param end int koncov index (nen do vsledku zahrnut)
   * @return ComplexVector vsledn vektor
   */
  public ComplexVector subVector(int start, int end) {
    if(end > start) {
      if(start < 0) start = 0;
      if(end >= length) end = length - 1;

      ComplexVector res = new ComplexVector(end - start);
      for (int i = start; i < end; i++) {
        res.num[i-start] = new Complex(this.get(i));
      }
      return res;
    }
    return new ComplexVector(0);
  }

  /**
   * Vrac vektor druhch mocnin absolutnch hodnot komplexnch sel -> r<sup>2</sup> + i<sup>2</sup>
   * @return Vektor
   */
  public Vektor getPowerSpectrum() {
    Vektor res = new Vektor(length);
    for(int i = length-1; i >= 0; i--) {
      res.set(i, num[i].getIm()*num[i].getIm() + num[i].getReal()*num[i].getReal());
    }
    return res;
  }

  public String toString() {
    StringBuffer buf = new StringBuffer();
    for(int i = 0; i < length; i++) {
      buf.append(num[i]);
      buf.append('\n');
    }
    return buf.toString();
  }
}
