/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package mathutil;

/**
 * <p>Tda implementuje filtraci dat.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class Filter {
  private Filter() {}

  /**
   * Pole relnch sel je filtrovno filtrem urenm vektorem koeficient
   * @param row double[] data k filtrovn
   * @param koefs Vektor vektor koeficient filtru
   * @param begin int index vektoru, kter je povaovn za sted filtru
   * @return double[]
   */
  public static double[] filter(double[] row, Vektor koefs, int begin) {
    double[] res = new double[row.length];

    for (int i = 0; i < begin; i++) {
      double sum = 0;
      for (int j = 0; j < koefs.getLength(); j++) {
        if (i + j < begin) {
          sum += koefs.get(j) * (2 * row[0] - row[begin - (i + j)]);
        }
        else {
          sum += koefs.get(j) * row[i + j - begin];
        }
      }
      res[i] = sum;
    }

    int length = row.length - begin;
    for (int i = begin; i < length; i++) {
      double sum = 0;
      for (int j = 0; j < koefs.getLength(); j++) {
        sum += row[i + j - begin] * koefs.get(j);
      }
      res[i] = sum;
    }

    for (int i = length; i < row.length; i++) {
      double sum = 0;
      for (int j = 0; j < koefs.getLength(); j++) {
        if (i + j - begin >= row.length) {
          sum += koefs.get(j) *
              (2 * row[row.length - 1] - row[2 * row.length - i - j + begin - 2]);
        }
        else {
          sum += koefs.get(j) * row[i + j - begin];
        }
      }
      System.out.println("");
      res[i] = sum;
    }

    return res;
  }

  /**
   * Pole relnch sel je filtrovno filtrem urenm matic koeficient.
   * Matice by mla bt symetrick.
   * begin by mla bt doln cel st velikosti matice.
   * @param row double[] data k filtrovn
   * @param koefs Matrix matice filtru
   * @param begin int dek matice, kter je povaovn za sted filtru
   * @return double[]
   */
  public static double[] filter(double[] row, Matrix koefs, int begin) {
    double[] res = new double[row.length];

    if(row.length > koefs.getWidth()) {
      for (int i = 0; i < begin; i++) {
        double sum = 0;
        for (int j = 0; j < koefs.getWidth(); j++) {
          sum += row[j] * koefs.get(j, i);
        }
        res[i] = sum;
      }

      int length = row.length - koefs.getHeight() + begin;
      for (int i = begin; i < length; i++) {
        double sum = 0;
        for (int j = 0; j < koefs.getWidth(); j++) {
          sum += row[i + j - begin] * koefs.get(j, begin);
        }
        res[i] = sum;
      }

      for (int i = length; i < row.length; i++) {
        double sum = 0;
        for (int j = 0; j < koefs.getWidth(); j++) {
          sum += row[row.length - koefs.getWidth() +
              j] * koefs.get(j, i - length + begin);
        }
        res[i] = sum;
      }
    } else {
      System.out.println(row.length + " : " + koefs.getWidth());
    }

    return res;
  }
}
