/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package mathutil;

/**
 * <p>Tda implementuje jednoduch matematick funkce.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class MathUtil {
  /**
   * Funkce pevd posloupnost x, y na hel, kter jednotliv body svraj s osou x
   * @param x double[]
   * @param y double[]
   * @param unwrap boolean jestlie je true, elimnuje skoky vt ne PI
   * @return double[]
   */
  public static double[] getFi(double x[], double y[], boolean unwrap) {
    int length = Math.min(x.length, y.length);
    double[] res = new double[length];
    double lastAngle = 0;
    if(length > 2) {
      for (int i = 1; i < length; i++) {
        double ang = Math.atan2(y[i], x[i]);
        ang = (ang < 0) ? (ang + 2 * Math.PI) : ang;
        if (unwrap) {
          if ( (ang - lastAngle) < -Math.PI) {
            lastAngle -= 2 * Math.PI;
          }
          else if ( (ang - lastAngle) > Math.PI) {
            lastAngle += 2 * Math.PI;
          }
          res[i] = res[i - 1] + ang - lastAngle;
          lastAngle = ang;
        }
        else {
          res[i] = ang;
        }
      }
      res[0] = res[1];
      res[length - 1] = res[length - 1];
    }
    return res;
  }
}
