/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.data.doubleViewers;

import spirals.data.viewers.TabletDataViewer;
import spirals.motol.database.Data;
import javax.swing.*;
import java.awt.*;

/**
 * <p>Zkladn abstraktn tda pro zobrazovn dat ve dvou oknech.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public abstract class DoubleViewer
    extends TabletDataViewer {
  /**
   * horn okno
   */
  protected DoubleViewerTop topViewer;
  protected JPanel topControls = new JPanel();
  protected JPanel top = new JPanel() {
    public void paintComponent(Graphics g) {
      topViewer.paint(g);
    }
  };

  /**
   * spodn okno
   */
  protected DoubleViewerBottom bottomViewer;
  protected JPanel bottom = new JPanel() {
    public void paintComponent(Graphics g) {
      bottomViewer.paint(g);
    }
  };

  public DoubleViewer(JPanel screen, JPanel controls, Data data, boolean all, boolean showUp) {
    super(screen, controls, data, all, showUp);
    this.install();
  }

  private void install() {
    top.setBorder(BorderFactory.createEtchedBorder());
    bottom.setBorder(BorderFactory.createEtchedBorder());
    screen.setLayout(new GridLayout(2, 1));
    screen.setBorder(BorderFactory.createEmptyBorder());
    JPanel up = new JPanel(new BorderLayout());
    topControls.setLayout(new BoxLayout(topControls, BoxLayout.Y_AXIS));
    topControls.setBorder(BorderFactory.createEmptyBorder(0, 10, 0, 0));
    up.add(top, BorderLayout.CENTER);
    up.add(topControls, BorderLayout.EAST);
    screen.add(up);
    screen.add(bottom);
  }

  public void paint(Graphics g) {
    g.setColor(Color.BLACK);
    g.fillRect(0, 0, screen.getWidth(), screen.getHeight());
    top.repaint();
    bottom.repaint();
  }
}
