/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.data.viewers;

import spirals.motol.database.Data;
import java.awt.*;
import java.awt.event.*;
import mathutil.*;
import javax.swing.*;
import javax.swing.event.*;
import spirals.data.TabletData;
import java.text.ParseException;

/**
 * <p>Tda roziuje prohle se zoomem.</p>
 * <p>Zobrazovan data vyfiltruje SG filtrem v zvislosti na prmrn rychlosti kreslen.
 * Pvodn a filtrovan data jsou zobrazena pes sebe.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class FiltWidthTabletDataViewer
    extends ZoomTabletDataViewer {
  /**
   * polomr psu pro zobrazen filtrovan posloupnosti
   */
  private double offset = .1; // in cm
  /**
   * rozlien tabletu
   */
  private final double dpi = 1000;
  private final int resampleRate = 50;
  private int len;
  private TabletData filtered;
  private double filtX[];
  private double filtY[];
  private double fi[];
  private boolean toBig[];
  private JSpinner fieldLength;
  private JSpinner fieldOffset;

  public FiltWidthTabletDataViewer(JPanel screen, JPanel controls, Data data,
                                   boolean all, boolean showUp) {
    super(screen, controls, data, all, showUp);
    this.install();
  }

  private void install() {
    fieldLength = new JSpinner(new SpinnerNumberModel(len, 10, 300, 10));
    fieldLength.addKeyListener(new KeyAdapter() {
      public void keyPressed(KeyEvent e) {
        if (e.getKeyCode() == e.VK_ENTER) {
          lengthChange();
        }
      }
    });
    fieldLength.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        lengthChange();
      }
    });
    fieldLength.setPreferredSize(new Dimension(50,
                                               (int) fieldLength.
                                               getPreferredSize().
                                               getHeight()));
    controls.add(new Label("Filtr:"));
    controls.add(fieldLength);

    fieldOffset = new JSpinner(new SpinnerNumberModel(offset, .1, 2, .1));
    fieldOffset.addKeyListener(new KeyAdapter() {
      public void keyPressed(KeyEvent e) {
        if (e.getKeyCode() == e.VK_ENTER) {
          offsetChange();
        }
      }
    });
    fieldOffset.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        offsetChange();
      }
    });
    fieldOffset.setPreferredSize(new Dimension(50,
                                               (int) fieldOffset.
                                               getPreferredSize().
                                               getHeight()));
    controls.add(new Label("ka:"));
    controls.add(fieldOffset);
    controls.add(new Label("cm"));
  }

  /**
   * nastav novou dlku filtru
   */
  private void lengthChange() {
    try {
      fieldLength.commitEdit();
    }
    catch (ParseException ex) {}
    len = Integer.parseInt(fieldLength.getValue().toString());
    setFiltXY();
    screen.repaint();
  }

  /**
   * nastav nov rozpt
   */
  private void offsetChange() {
    try {
      fieldOffset.commitEdit();
    }
    catch (ParseException ex) {}
    offset = Double.parseDouble(fieldOffset.getValue().toString());
    screen.repaint();
  }

  /**
   * filtruje data SG filtrem dan dlky
   */
  private void setFiltXY() {
    filtered = tabletData.resampleEquidistant(resampleRate);
    double[] oldX = filtered.getDoubleX();
    double[] oldY = filtered.getDoubleY();
    filtX = SavitzkyGolayFilter.filterAdaptive(oldX, 2, len);
    filtY = SavitzkyGolayFilter.filterAdaptive(oldY, 2, len);

    int size = filtX.length;
    double dx[] = new double[size];
    double dy[] = new double[size];
    for (int i = 1; i < size - 1; i++) {
      dx[i] = (double) (filtX[i - 1] - filtX[i + 1]) / 2;
      dy[i] = (double) (filtY[i - 1] - filtY[i + 1]) / 2;
    }
    fi = MathUtil.getFi(dx, dy, false);

    toBig = new boolean[size];
    double limit = offset * dpi;
    if(limit == 0) limit = .1 * dpi; // offset jeste neni inicializovany
    for(int i = 0; i < size; i++) {
      double d = Math.sqrt((filtX[i] - oldX[i])*(filtX[i] - oldX[i]) + (filtY[i] - oldY[i])*(filtY[i] - oldY[i]));
      if(d > limit) {
        toBig[i] = true;
      }
      else {
        toBig[i] = false;
      }
    }
  }

  public void paint(Graphics g) {
    this.clearWindow(g);
    this.drawTablet(g);
    g.setColor(Color.RED);
    g.setColor(Color.ORANGE);
    this.paintSpiral(g, 0);
    g.setColor(Color.ORANGE);
    this.paintSpiral(g, -offset);
    this.paintSpiral(g, offset);
    this.paintData(g, filtered);
  }

  /**
   * kresl filtrvan obrazec s danm posunutm
   * @param g Graphics
   * @param offset double posunut jednotlivch bod po ten
   */
  protected void paintSpiral(Graphics g, double offset) {
    int size = fi.length - 1;

    int x, ox = 0;
    int y, oy = 0;

    double koef = this.getKoeficient();
    int scrHeight = screen.getHeight();
    int height = scrHeight - this.getTabletOffsetY() -
        (int) (this.getOffsetY() * koef);
    int left = this.getTabletOffsetX() - (int) (this.getOffsetX() * koef);
    int offsetX = this.getOffsetX();
    int offsetY = this.getOffsetY();

    for (int i = 0; i < size; i++) {
      x = (int) ( (filtX[i] + offsetX + offset * dpi * Math.sin(fi[i])) * koef);
      y = (int) ( (filtY[i] + offsetY - offset * dpi * Math.cos(fi[i])) * koef);
      x = x + left;
      y = height - y;
      if (i != 0) {
        g.drawLine(ox, oy, x, y);
      }

      ox = x;
      oy = y;
    }
  }

  protected void setDrawColor(Graphics g, int index) {
    if(toBig[index]) {
      g.setColor(Color.RED);
    } else {
      g.setColor(Color.BLACK);
    }
  }

  public void updateData() {
    super.updateData();
    len = (int) (1.1 *
                 (tabletData.getMaxX() + tabletData.getMaxY() -
                  tabletData.getMinX() - tabletData.getMinY()) / 200);
    this.setFiltXY();
  }

  public String getDescription() {
    return
        "Daty je proloena spirla. Data se maj vejt do psu irokho 1 cm";
  }
}
