/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.data.viewers;

import java.awt.*;
import javax.swing.*;
import tablet.*;
import spirals.data.TabletData;
import spirals.motol.database.Data;

/**
 * <p>Nejjednosu tda pro zobrazovn dat.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class SimpleTabletDataViewer
    extends TabletDataViewer {
  /**
   * number of pixels between active area and border
   */
  protected final int border = 4;

  public SimpleTabletDataViewer(JPanel screen, JPanel controls, Data data,
                                boolean all, boolean showUp) {
    super(screen, controls, data, all, showUp);
    this.install();
  }

  private void install() {
    screen.setBorder(BorderFactory.createEtchedBorder());
  }

  /**
   * nartne tablet a vykresl data
   * @param g Graphics
   */
  public void paint(Graphics g) {
    this.clearWindow(g);
    this.drawTablet(g);
    this.paintData(g, tabletData);
  }

  /**
   * vymae okno
   * @param g Graphics
   */
  protected void clearWindow(Graphics g) {
    g.setColor(Color.LIGHT_GRAY);
    g.fillRect(0, 0, screen.getWidth(), screen.getHeight());
  }

  /**
   * vykresl obrysy tabletu
   * @param g Graphics
   */
  protected void drawTablet(Graphics g) {
    g.setColor(Color.WHITE);
    g.fillRect(this.getTabletOffsetX(),
               this.getTabletOffsetY(),
               this.getPictureWidth(),
               this.getPictureHeight());
  }

  /**
   * vlastn kreslen dat
   * @param g Graphics
   */
  protected void paintData(Graphics g, TabletData tabletData) {
    // initialize for drawing
    int x, ox = 0;
    int y, oy = 0;
    boolean pen_down = false;

    double koef = this.getKoeficient(); // pomr velikosti viewportu k tabletu
    int scrHeight = screen.getHeight();
    int scrWidth = screen.getWidth();
    int height = scrHeight - this.getTabletOffsetY() -
        (int) (this.getOffsetY() * koef); // spodn okraj na obrazovce odpovdajc nejvymu bodu tabletu
    int left = this.getTabletOffsetX() - (int) (this.getOffsetX() * koef); // lev okraj na obrazovce odpovdajc nejlevjmu bodu tabletu
    int offsetX = this.getOffsetX();
    int offsetY = this.getOffsetY();

    // vykresli vechny body tabletu
    for (int i = 0; i < tabletData.size(); i++) {
      TabletPoint point = tabletData.get(i);
      x = (int) ( (point.getX() + offsetX) * koef);
      y = (int) ( (point.getY() + offsetY) * koef);
      x = x + left;
      y = height - y;

      if (!point.isTouched() && showUp) {
        pen_down = false;
        continue;
      }

      if (!pen_down) { // beginning of new line
        pen_down = true;
        ox = x;
        oy = y;
      }

      // nekresli co nen vidt
      if (! ( ( (x < 0) && (ox < 0)) ||
             ( (y < 0) && (oy < 0)) ||
             ( (x > scrWidth) && (ox > scrWidth)) ||
             ( (y > scrHeight) && (oy > scrHeight)))) {
        this.setDrawColor(g, i);
        g.drawLine(x, y, ox, oy);
      }

      ox = x;
      oy = y;
    }
  }

  protected void setDrawColor(Graphics g, int index) {
    g.setColor(Color.BLACK);
  }

  /**
   * @return int ka viewportu
   */
  protected int getScreenWidth() {
    return screen.getWidth() - 2 * border;
  }

  /**
   * @return int vka viewportu
   */
  protected int getScreenHeight() {
    return screen.getHeight() - 2 * border;
  }

  /**
   * @return int pomr velikosti viewportu k tabletu
   */
  protected double getKoeficient() {
    double r_w = ( (double)this.getScreenWidth()) / (double) ti.getWidth();
    double r_h = ( (double)this.getScreenHeight()) / (double) ti.getHeight();
    return (r_h > r_w) ? r_w : r_h;
  }

  /**
   * @return int ka tabletu v obrazovkovch bodech
   */
  protected int getPictureWidth() {
    return (int) (ti.getWidth() * this.getKoeficient());
  }

  /**
   * @return int vka tabletu v obrazovkovch bodech
   */
  protected int getPictureHeight() {
    return (int) (ti.getHeight() * this.getKoeficient());
  }

  /**
   * vrac slo, kde na pltn je nejlevj bod tabletu
   * @return int
   */
  protected int getTabletOffsetX() {
    return (this.getScreenWidth() - this.getPictureWidth()) / 2 + border;
  }

  /**
   * vrac slo, kde na pltn je nejvy bod tabletu
   * @return int
   */
  protected int getTabletOffsetY() {
    return (this.getScreenHeight() - this.getPictureHeight()) / 2 + border;
  }

  /**
   * posunut tabletu v souadnicch tabletu
   * @return int
   */
  protected int getOffsetX() {
    return 0;
  }

  /**
   * posunut tabletu v souadnicch tabletu
   * @return int
   */
  protected int getOffsetY() {
    return 0;
  }

  public String getDescription() {
    return "Jednoduch prohle dat - nic neum";
  }
}
