/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.data.viewers;

import javax.swing.JPanel;
import java.awt.*;
import spirals.motol.database.Data;
import spirals.data.DataComboBox;
import mathutil.*;

/**
 * <p>Prohle vykresl spektrogram vypotan z dat.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class SpecgramTabletDataViewer
    extends SimpleTabletDataViewer {
  private DataComboBox dcb;
  private Matrix specgram;
  private static final int palleteSize = 768;
  /**
   * barevn paleta pro zobrazovn: ern -> modr -> oranov -> erven
   */
  private static final Color[] pallete = new Color[palleteSize];
  static {
    for (int i = 0; i < 256; i++) {
      pallete[i] = new Color(0, 0, i);
    }
    for (int i = 256; i < 512; i++) {
      pallete[i] = new Color(i - 256, i - 256, 511 - i);
    }
    for (int i = 512; i < 768; i++) {
      pallete[i] = new Color(255, 767 - i, 0);
    }
  }

  public SpecgramTabletDataViewer(JPanel screen, JPanel controls, Data data,
                                  boolean all, boolean showUp) {
    super(screen, controls, data, all, showUp);
    this.install();
  }

  private void install() {
    dcb = new DataComboBox(this, data, all);
    controls.add(dcb);
    this.updateData();
  }

  public void paint(Graphics g) {
    this.clearWindow(g);
    double kw = this.getScreenWidth() / (double) specgram.getWidth();
    int w = (int) Math.ceil(kw);
    double kh = this.getScreenHeight() / (double) specgram.getHeight();
    int h = (int) Math.ceil(kh);
    for (int i = 0; i < specgram.getWidth(); i++) {
      for (int j = 0; j < specgram.getHeight(); j++) {
        int index = (int) (specgram.get(i, j) * dcb.getKoeficient());
        if (index < 0) {
          index = 0;
        }
        if (index >= palleteSize) {
          index = palleteSize - 1;
        }
        g.setColor(pallete[index]);
        g.fillRect( (int) (i * kw) + border, (int) (j * kh) + border, w, h);
      }
    }
    this.paintBar(g);
  }

  private void paintBar(Graphics g) {
    int k = (int) Math.floor( (double) palleteSize /
                             (this.getScreenHeight() * 0.6));
    for (int i = 0; i < palleteSize; i += k) {
      g.setColor(pallete[i]);
      g.drawLine(this.getScreenWidth() - 15, this.getScreenHeight() - i / k,
                 this.getScreenWidth() - 5, this.getScreenHeight() - i / k);
    }
  }

  /**
   * vypot spektrogram z rozdlu hl pvodnho a filtrovanho
   */
  public void updateData() {
//    super.updateData();
    specgram = Spectrogram.specgram(new ComplexVector(this.getFiltFi()), 64,
                                    256, 64, 4);
  }

  /**
   * Vypot filtrovan vektor hl, kter body svraj s osou x
   * @return Vektor rozdl pvodnho a filtrovanho vektoru
   */
  private Vektor getFiltFi() {
    if(dcb != null) {
      int len = 2 * (int) Math.ceil(700 / tabletData.getMeanVelocityUpdated()) + 1;
      double data[] = dcb.getData();
      double filtData[] = SavitzkyGolayFilter.filter(data, 1, len);
//    double filtData[] = SavitzkyGolayFilter.filterAdaptive(data, 1, len);
      return new Vektor(data).substract(new Vektor(filtData));
    } else {
      return new Vektor();
    }
  }

  public String getDescription() {
    return "Data jsou zobrazena ve spektrogramu";
  }
}
