/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.data.viewers;

import spirals.data.*;
import spirals.motol.database.Data;
import java.awt.Graphics;
import javax.swing.JPanel;
import tablet.TabletInfo;

/**
 * <p>Zkladn abstraktn tda, kterou mus rozit kad zobrazova dat.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public abstract class TabletDataViewer {
  /**
   * active tablet measures
   */
  protected TabletInfo ti;
  /**
   * captured data
   */
  protected TabletData tabletData;
  /**
   * captured info about data
   */
  protected Data data;
  /**
   * panel to draw on
   */
  protected JPanel screen;
  /**
   * panel to place controls
   */
  protected JPanel controls;
  protected boolean all = false;
  protected boolean showUp = true;

  /**
   * creates new viewer
   * @param screen panel to draw on
   * @param controls panel to place controls
   * @param data captured data from tablet
   * @param showUp view only points where is pen down?
   */
  public TabletDataViewer(JPanel screen, JPanel controls, Data data,
                          boolean all, boolean showUp) {
    this.screen = screen;
    this.controls = controls;
    this.all = all;
    this.showUp = showUp;
    controls.removeAll(); // vyist ovladac panel
    screen.removeAll();
    this.setTabletData(data);
  }

  /**
   * derived class must override this method to implement drawing
   */
  public abstract void paint(Graphics g);

  /**
   * repaint provided panel with data
   */
  public final void repaint() {
    screen.repaint();
  }

  /**
   * provides new TabletData for viewing
   */
  public final void setTabletData(Data data) {
    this.data = data;
    if (all) {
      this.tabletData = data.getAllData();
    }
    else {
      this.tabletData = data.getData();
    }
    this.ti = data.getTabletInfo();
    this.updateData();
    this.updateControls();
  }

  /**
   * sets new tablet measures
   */
  public final void setTabletInfo(TabletInfo tabletInfo) {
    this.ti = tabletInfo;
    this.repaint();
  }

  /**
   * vrt textov popis prohlee
   * @return String
   */
  public String getDescription() {
    return "";
  }

  /**
   * Vol se vdy pi zmn dat. Uprav ovldac prvky prohlee
   */
  public void updateControls() {}

  /**
   * Zamez pouvn ovldacch prvk
   */
  public void disableControls() {}

  /**
   * Vol se vdy pi zmn dat. Vechny vpoty nad daty by mly bt volny z tto funkce
   */
  public void updateData() {}
}
