/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.database;

import spirals.motol.loaders.DataLoader;
import spirals.data.TabletData;
import java.lang.ref.SoftReference;
import tablet.*;
import java.text.*;
import java.util.Date;
import java.util.List;

/**
 * <p>Class contains information about tablet and about captured data.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @see spirals.data.TabletData TabletData
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class Data {
  /**
   * nvtva, ke kter data pat
   */
  private Visit visit = null;
  /**
   * tda pro nahrn dat z databze
   */
  private DataLoader loader = null;
  /**
   * ukazatel na vechna nasnman data
   */
  private SoftReference dataAll = null;
  /**
   * ukazatel na vybran data
   */
  private SoftReference data = null;
  /**
   * informace o tabletu, kterm byla data nasnmna
   */
  private TabletInfo ti = null;
  /**
   * datum pozen dat
   */
  private Date date = null;
  /**
   * nzev dat
   */
  private String description = null;
  /**
   * doplnujc informace
   */
  private String memo = null;
  /**
   * zatek vybranho seku dat
   */
  private int start;
  /**
   * konec vybranho seku dat
   */
  private int end;

  /**
   * vytvo nov data
   * @param visit Visit
   */
  public Data(Visit visit) {
    this(visit, visit.getLoader().getDefaultDataLoader());
    this.date = new Date();
    this.ti = Engine.getInstance().getTablet().getTabletInfo();
//    this.dataAll = new SoftReference(new TabletData(this));
  }

  protected Data(Visit visit, DataLoader loader) {
    this.visit = visit;
    this.loader = loader;
  }

  /**
   * nahraje data identifikovan datem
   * @param visit Visit
   * @param date Date
   */
  public Data(Visit visit, Date date) {
    this(visit, visit.getLoader().getDefaultDataLoader(), date);
  }

  /**
   * nahraje data identifikovan datem
   * @param visit Visit
   * @param loader DataLoader
   * @param date Date
   */
  public Data(Visit visit, DataLoader loader, Date date) {
    this(visit, loader);
    this.date = date;
    loader.load(this, visit, date);
  }

  public TabletInfo getTabletInfo() {
    return ti;
  }

  public void setTabletInfo(TabletInfo tabletInfo) {
    ti = tabletInfo;
  }

  public void setMemo(String memo) {
    this.memo = memo;
  }

  public String getMemo() {
    return memo;
  }

  public void setDescription(String description) {
    if (this.description != null) {
      visit.getTestList().updateHist(this.description, description);
    }
    this.description = description;
  }

  public String getDescription() {
    return description;
  }

  public void setStart(int start) {
    this.data = null;
    this.start = start;
  }

  public int getStart() {
    return start;
  }

  public void setEnd(int end) {
    this.data = null;
    this.end = end;
  }

  public int getEnd() {
    return end;
  }

  public void setDate(Date date) {
    this.date = date;
  }

  public Date getDate() {
    return date;
  }

  public Visit getVisit() {
    return visit;
  }

  public void setVisit(Visit visit) {
    this.visit = visit;
    this.loader = visit.getLoader().getDefaultDataLoader();
  }

  public void setLoader(DataLoader loader) {
    this.loader = loader;
  }

  public void save(boolean saveData) {
    visit.addData(this);
    loader.save(this, saveData);
  }

  public boolean equals(Object o) {
    if (o instanceof Data) {
      Data d = (Data) o;
      return date.equals(d.date);
    }
    return false;
  }

  public String toString() {
    SimpleDateFormat df = new SimpleDateFormat("dd. MM. yyyy, HH:mm:ss");
    return description + " (" + df.format(date) + ")";
  }

  /**
   * Vrac vechna nasnman data
   * @return TabletData
   */
  public TabletData getAllData() {
    TabletData td = (this.dataAll == null) ? null :
        (TabletData) (this.dataAll.get());
    if (td == null) {
      td = loader.getData(this);
      this.dataAll = new SoftReference(td);
    }
    return td;
  }

  /**
   * Vrac vybranou st dat
   * @return TabletData
   */
  public TabletData getData() {
    TabletData td = (this.data == null) ? null : (TabletData) (this.data.get());
    if (td == null) {
      List<TabletPoint> tdl = this.getAllData().getData();
      td = new TabletData(this);
      if (start < 0) {
        start = 0;
      }
      if (end > tdl.size()) {
        end = tdl.size();
      }
      for (int i = start; i < end; i++) {
        td.add(tdl.get(i));
      }
      this.data = new SoftReference(td);
    }
    return td;
  }

  public int size() {
    return this.getData().size();
  }
}
