/*      Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.gui;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import spirals.motol.database.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Date;

/**
 * <p>Panel with informations about pacient</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.2.0
 */
public class PacientPanel
    extends RemovablePanel {
  /**
   * pacient for this panel
   */
  private Pacient pac;
  /**
   * main frame
   */
  private MainFrame mf;
  /**
   * index of selected visit
   */
  private int index = -1;
  /**
   * show visit button
   */
  private JButton selectVisit;

  /**
   * listen for key events on form fields
   */
  private KeyListener changeKeyAction = new KeyAdapter() {
    public void keyTyped(KeyEvent e) {
      PacientPanel.this.setChanged(true);
    }
  };
  /**
   * listen for action events on form fields
   */
  private ChangeListener changeListener = new ChangeListener() {
    public void stateChanged(ChangeEvent e) {
      PacientPanel.this.setChanged(true);
    }
  };

  /**
   * listen for action events on form fields
   */
  private ActionListener changeAction = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      PacientPanel.this.setChanged(true);
    }
  };

  /**
   * main content of panel
   */
  private JPanel mainContent = new JPanel(new GridBagLayout());
  /**
   * text field for surname
   */
  private JTextField fieldSurname;
  /**
   * text field for firstname
   */
  private JTextField fieldFirstname;
  /**
   * formatted text field for born date
   */
  private JSpinner fieldBornDate = new JSpinner(new SpinnerDateModel());
  private JSpinner.DateEditor dateEditor = new JSpinner.DateEditor(
      fieldBornDate, "dd.MM.yyyy");

  private JFormattedTextField fieldID;
  /**
   * options for right-hand question
   */
  private String[] opt = {
      "Pravk", "Levk"};
  /**
   * right-hand question
   */
  private JComboBox fieldRightHand = new JComboBox(opt);
  /**
   * change record button
   */
  JButton changeButton = new JButton("Zm daje");

  /**
   * create PacientPanel for Pacient
   * @param frame MainFrame
   * @param pac Pacient
   * @param changable user can change fields
   */
  public PacientPanel(MainFrame frame, final Pacient pac, boolean changable) {
    super(new BorderLayout());
    this.mf = frame;
    this.pac = pac;
    try {
      DefaultFormatter IDFormatter = new MaskFormatter("###### / ###*");
      fieldID = new JFormattedTextField(IDFormatter);
    }
    catch (Exception e) {}
    this.install();
    this.setChangable(changable);
    this.setChanged(changable);
    /*    if(changaable) {
          this.saveData();
          SwingUtilities.invokeLater(new Runnable() {
            public void run() {
              mf.getTree().getTree().updateUI();
              mf.getTree().scrollTreePath(pac);
              fieldFirstname.requestFocusInWindow();
            }
          });
        }*/
  }

  /**
   * create lay out for panel
   */
  private void install() {
    this.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

    fieldSurname = new JTextField(pac.getSurname());
    fieldSurname.addKeyListener(changeKeyAction);
    fieldFirstname = new JTextField(pac.getFirstname());
    fieldFirstname.addKeyListener(changeKeyAction);
    fieldBornDate.setValue(pac.getBornDate());
    fieldBornDate.addKeyListener(changeKeyAction);
    String ID = pac.getID();
    if (ID.length() > 0) {
      ID = ID.substring(0, 6) + " / " + ID.substring(6);
    }
    fieldID.setValue(ID);
    fieldID.addKeyListener(changeKeyAction);
    fieldBornDate.setEditor(dateEditor);
    fieldBornDate.addChangeListener(changeListener);
    fieldRightHand.setSelectedIndex(pac.isRightHanded() ? 0 : 1);
    fieldRightHand.addActionListener(changeAction);
    JList visitList = new JList(pac.getVisits().toArray());
    visitList.getSelectionModel().setSelectionMode(ListSelectionModel.
        SINGLE_SELECTION);
    visitList.setFont(mf.getDefaultFont());
    visitList.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
        index = e.getFirstIndex();
        selectVisit.setEnabled(true);
      }
    });
    visitList.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        if ( (e.getButton() == e.BUTTON1) && (e.getClickCount() > 1)) {
          selectVisit();
        }
      }
    });
    selectVisit = new JButton("Zobraz nvtvu");
    selectVisit.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        selectVisit();
      }
    });
    selectVisit.setEnabled(false);
    JButton newVisit = new JButton("Nov nvtva");
    newVisit.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (save()) {
          Visit visit = pac.visitExists(new Visit(pac, pac.getLastVisit()));
          mf.setRightPanel(new VisitPanel(mf, visit, true));
        }
        else {
          JOptionPane.showMessageDialog(mf, "Pacient nem platn daje",
                                        "Chyba", JOptionPane.ERROR_MESSAGE);
        }
      }
    });
    JPanel bottom = new JPanel();
    bottom.add(newVisit);
    bottom.add(selectVisit);

    this.place(1, new JLabel("Jmno:"), fieldFirstname);
    this.place(2, new JLabel("Pjmen:"), fieldSurname);
    this.place(3, new JLabel("Datum narozen:"), fieldBornDate);
    this.place(4, new JLabel("Rodn slo:"), fieldID);
    this.place(5, new JLabel("Pravk/levk:"), fieldRightHand);
    this.place(6, new JLabel(" "));
    this.place(7, new JLabel("Nvtvy"), Box.createHorizontalBox());
    this.place(8, new JScrollPane(visitList), Box.createHorizontalBox(), 100);
    this.place(9, bottom);

    this.add(mainContent, BorderLayout.NORTH);
    JPanel bottomPanel = new JPanel(new BorderLayout());
    changeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        setChangable(true);
      }
    });
    bottomPanel.add(changeButton, BorderLayout.WEST);
    this.add(bottomPanel, BorderLayout.SOUTH);
  }

  /**
   * place two components in gridbag layout on main panel
   * @param row row number in parent component
   * @param c1 first component to place
   * @param c2 second component to place
   */
  private void place(int row, JComponent c1, JComponent c2) {
    mainContent.add(c1,
                    new GridBagConstraints(0, row, 1, 1, 0.0, 0.0,
                                           GridBagConstraints.WEST,
                                           GridBagConstraints.BOTH,
                                           new Insets(0, 0, 0, 10), 0, 0));
    mainContent.add(c2,
                    new GridBagConstraints(1, row, 2, 1, 0.5, 0.0,
                                           GridBagConstraints.CENTER,
                                           GridBagConstraints.BOTH,
                                           new Insets(0, 0, 0, 0), 0, 0));
    mainContent.add(Box.createHorizontalBox(),
                    new GridBagConstraints(3, row, 2, 1, 1.0, 0.0,
                                           GridBagConstraints.CENTER,
                                           GridBagConstraints.BOTH,
                                           new Insets(0, 0, 0, 0), 0, 0));
  }

  /**
   * place two components in gridbag layout on main panel with gap above
   * @param row row number in parent component
   * @param c1 first component to place
   * @param c2 second component to place
   * @param space pixels of gap
   */
  private void place(int row, JComponent c1, JComponent c2, int space) {
    mainContent.add(c1,
                    new GridBagConstraints(0, row, 3, 1, 0.5, 0.0,
                                           GridBagConstraints.WEST,
                                           GridBagConstraints.BOTH,
                                           new Insets(5, 0, 0, 0), 0, 0));
    mainContent.add(c2,
                    new GridBagConstraints(1, row, 2, 1, 1.0, 1.0,
                                           GridBagConstraints.CENTER,
                                           GridBagConstraints.BOTH,
                                           new Insets(space, 0, 0, 0), 0, 0));
  }

  /**
   * place one component in gridbag layout on main panel across all width
   * @param row row number in parent component
   * @param c component to place
   */
  private void place(int row, JComponent c) {
    mainContent.add(c,
                    new GridBagConstraints(0, row, 3, 1, 0.0, 0.0,
                                           GridBagConstraints.EAST,
                                           GridBagConstraints.NONE,
                                           new Insets(10, 0, 0, 0), 0, 0));
    mainContent.add(Box.createHorizontalBox(),
                    new GridBagConstraints(3, row, 2, 1, 1.0, 0.0,
                                           GridBagConstraints.CENTER,
                                           GridBagConstraints.BOTH,
                                           new Insets(0, 0, 0, 0), 0, 0));
  }

  /**
   * set form fields and button changable
   */
  private void setChangable(boolean changable) {
    fieldSurname.setEnabled(changable);
    fieldFirstname.setEnabled(changable);
    fieldBornDate.setEnabled(changable);
    fieldID.setEnabled(changable);
    fieldRightHand.setEnabled(changable);
    changeButton.setEnabled(!changable);
  }

  /**
   * called when panel is removed and changed
   */
  protected boolean remove() {
    return this.saveData();
  }

  /**
   * stores pacient to disk
   */
  private boolean saveData() {
    if (save()) {
      return true;
    }
    return JOptionPane.showConfirmDialog(mf,
                                         "Pi pokraovn nebude pacient uloen do databze.\nChcete pokraovat?",
                                         "Rodn lso nebylo zadno",
                                         JOptionPane.YES_NO_OPTION) ==
        JOptionPane.YES_OPTION;
  }

  /**
   * pokus se uloit pacienta
   * @return boolean
   */
  private boolean save() {
    try {
      String ID = (String) fieldID.getValue();
      ID = ID.substring(0, 6) + ID.substring(9);
      pac.setID(ID);
      pac.setBornDate( (java.util.Date) fieldBornDate.getValue());
      pac.setFirstname(fieldFirstname.getText());
      pac.setSurname(fieldSurname.getText());
      pac.setRightHanded(fieldRightHand.getSelectedIndex() == 0 ? true : false);
      if (pac.save()) {
        return true;
      }
    }
    catch (Exception e) {}
    return false;
  }

  /**
   * Do pravho panelu umst vybranou nvtvu
   */
  public void selectVisit() {
    if (index >= 0) {
      Visit visit = pac.getVisit(index);
      mf.setRightPanel(new VisitPanel(mf, visit, false));
    }
  }
}
