/*      Author: Miroslav Skrbek, Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2006-12-24
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License v2 as published by the
 *	Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package tablet;

/**
 * <p>Class stores information about tablet</p>
 *
 * <p>Copyright: Copyright (c) 2005 Miroslav Skrbek, Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Miroslav Skrbek
 * @author Jan Dolezel
 * @version 1.2
 */
public class TabletInfo {
  /**
   * tablet name
   */
  private String name;
  /**
   * minimal x coordinate
   */
  private int minx;
  /**
   * maximal x coordinate
   */
  private int maxx;
  /**
   * minimal y coordinate
   */
  private int miny;
  /**
   * maximal y coordinate
   */
  private int maxy;

  /**
   * creates new tablet information with x and y minimal coordinates set to 0
   * @param name tablet name
   * @param maxx maximal x coordinate
   * @param maxy maximal y coordinate
   */
  public TabletInfo(String name, int maxx, int maxy) {
    this(name, 0, maxx, 0, maxy);
  }

  /**
   * creates new tablet information
   * @param name tablet name
   * @param minx minimal x coordinate
   * @param maxx maximal x coordinate
   * @param miny minimal y coordinate
   * @param maxy maximal y coordinate
   */
  public TabletInfo(String name, int minx, int maxx, int miny, int maxy) {
    this.name = name;
    this.minx = minx;
    this.miny = miny;
    this.maxx = maxx;
    this.maxy = maxy;
  }

  /**
   * returns tablet's name
   */
  public String getTabletName() {
    return name;
  }

  /**
   * returns tablet's minimal x coordinate
   */
  public int getMinX() {
    return minx;
  }

  /**
   * returns tablet's minimal y coordinate
   */
  public int getMinY() {
    return miny;
  }

  /**
   * returns tablet's maximal x coordinate
   */
  public int getMaxX() {
    return maxx;
  }

  /**
   * returns tablet's maximal y coordinate
   */
  public int getMaxY() {
    return maxy;
  }

  /**
   * returns tablet's width
   */
  public int getWidth() {
    return getMaxX() - getMinX() + 1;
  }

  /**
   * returns tablet's height
   */
  public int getHeight() {
    return getMaxY() - getMinY() + 1;
  }

  /**
   * returns string representation of tablet's naem and measures
   */
  public String toString() {
    return name + "(" + minx + "-" + maxx + ", " + miny + "-" + maxy + ")";
  }

  public boolean equals(Object o) {
    if(o instanceof TabletInfo) {
      TabletInfo ti = (TabletInfo) o;
      if((ti.maxx == maxx) && (ti.maxy == maxy) && (ti.minx == minx) && (ti.miny == miny) && (ti.name == name)) return true;
    }
    return false;
  }
}
